@@ -1,6 +1,6 @@
 /* OpenCP Module Player
  * copyright (c) 1994-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
- * copyright (c) 2004-'24 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2004-'26 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * PMain - main module (loads and inits all startup modules)
  *
@@ -1012,13 +1012,13 @@ static int init_modules(int argc, char *argv[])
 
 		if (epoch < 20240510)
 		{
-			fprintf (stderr, "ocp.ini update (0.2.110) add [modland.com] mirror=https://ftp.modland.com\n");
+			fprintf (stderr, "ocp.ini update (3.0.0) add [modland.com] mirror=https://ftp.modland.com\n");
 #ifdef _WIN32
-			fprintf (stderr, "ocp.ini update (0.2.110) add [modland.com] cachedir=$OCPDATAHOME\\modland.com\\\n");
+			fprintf (stderr, "ocp.ini update (3.0.0) add [modland.com] cachedir=$OCPDATAHOME\\modland.com\\\n");
 #else
-			fprintf (stderr, "ocp.ini update (0.2.110) add [modland.com] cachedir=$OCPDATAHOME/modland.com/\n");
+			fprintf (stderr, "ocp.ini update (3.0.0) add [modland.com] cachedir=$OCPDATAHOME/modland.com/\n");
 #endif
-			fprintf (stderr, "ocp.ini update (0.2.110) add [modland.com] showrelevantdirectoriesonly=1\n");
+			fprintf (stderr, "ocp.ini update (3.0.0) add [modland.com] showrelevantdirectoriesonly=1\n");
 
 			cfSetProfileString ("modland.com", "mirror", "https://ftp.modland.com");
 #ifdef _WIN32
@@ -1031,15 +1031,34 @@ static int init_modules(int argc, char *argv[])
 
 		if (epoch < 20240510)
 		{
-			fprintf(stderr, "ocp.ini update (0.2.107) add [libsidplayfp] filterrange6581=0.5\n");
-			cfSetProfileString  ("libsidplayfp", "filterrange6581", "0.5");
-			fprintf(stderr, "ocp.ini update (0.2.107) add [libsidplayfp] combinedwaveforms=Strong\n");
-			cfSetProfileString  ("libsidplayfp", "combinedwaveforms", "Strong");
+			fprintf(stderr, "ocp.ini update (0.2.107, revised in 3.0.0) add [libsidplayfp] filterrange6581=0.5\n");
+			cfSetProfileString ("libsidplayfp", "filterrange6581", "0.5");
+			fprintf(stderr, "ocp.ini update (0.2.107, revised in 3.0.0) add [libsidplayfp] combinedwaveforms=Strong\n");
+			cfSetProfileString ("libsidplayfp", "combinedwaveforms", "Strong");
 		}
 
-		if (epoch < 20240510)
+		if (epoch < 20250213)
 		{
-			cfSetProfileInt("version", "epoch", 20240510, 10);
+			fprintf(stderr, "ocp.ini update (3.1.0), updated comment for fontsize, adding 2=16x32\n");
+			cfSetProfileComment ("screen", "fontsize", "; if screentype=8: 0=8x8, 1=8x16, 2=16x32");
+		}
+
+		if (epoch < 20251026)
+		{
+			printf("ocp.ini update (3.1.0) removed [libsidplayfp] emulator=resid and filterbias=X, and added emulator=residfpII\n");
+			cfSetProfileString("libsidplayfp", "emulator", "residfpII");
+			cfRemoveEntry("libsidplayfp", "filterbias");
+		}
+
+		if (epoch < 20260111)
+		{
+			fprintf(stderr, "ocp.ini update (3.1.0) add [fscolors] QOA=6\n");
+			cfSetProfileInt ("fscolors", "QOA", 6, 10);
+		}
+
+		if (epoch < 20260111)
+		{
+			cfSetProfileInt("version", "epoch", 20260111, 10);
 			cfStoreConfig();
 			if (isatty(2))
 			{
@@ -1050,14 +1069,14 @@ static int init_modules(int argc, char *argv[])
 			sleep(5);
 		}
 	}
-	if (cfGetProfileInt("version", "epoch", 0, 10) != 20240510)
+	if (cfGetProfileInt("version", "epoch", 0, 10) != 20260111)
 	{
 		if (isatty(2))
 		{
-			fprintf(stderr,"\n\033[1m\033[31mWARNING, ocp.ini [version] epoch != 20240510\033[0m\n\n");
+			fprintf(stderr,"\n\033[1m\033[31mWARNING, ocp.ini [version] epoch != 20260111\033[0m\n\n");
 			sleep(5);
 		} else {
-			fprintf(stderr,"\nWARNING, ocp.ini [version] epoch != 20240510\n\n");
+			fprintf(stderr,"\nWARNING, ocp.ini [version] epoch != 20260111\n\n");
 		}
 	}
 
