@@ -20,10 +20,10 @@ struct mixchannel
 	uint16_t status;
 	union
 	{
-		uint32_t *voltabs[2];
 		int16_t vols[2];
 		float volfs[2];
 	} vol;
+	uint32_t *voltabs[2]; /* internal use by dev/mix.c and dev/mixasm.c */
 };
 
 struct cpifaceSessionAPI_t; /* cpiface.h */
@@ -48,6 +48,7 @@ extern const struct mixAPI_t *mixAPI;
 #define MIX_INTERPOLATE 32
 #define MIX_MAX 64
 #define MIX_PLAYFLOAT 128
-#define MIX_ALL 255
+#define MIX_PLAYSTEREO 256
+#define MIX_ALL 511
 
 #endif
