@@ -1,3 +1,23 @@
+/* OpenCP Module Player
+ * copyright (c) 2011-'26 François Revol <revol@free.fr>
+ *
+ * Common code for SDL Player device 1.x and 2.x
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ */
+
 static const struct plrDriverAPI_t *plrDriverAPI;
 
 static const struct plrDriver_t plrSDL;
@@ -21,7 +41,7 @@ static SDL_AudioDeviceID status;
 static int status;
 #endif
 
-void theRenderProc(void *userdata, Uint8 *stream, int len)
+static void theRenderProc(void *userdata, Uint8 *stream, int len)
 {
 	int pos1, length1, pos2, length2;
 
