@@ -1,5 +1,5 @@
 /* OpenCP Module Player
- * copyright (c) 2022 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2022-'26 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * FILE-IO handling for CDFS
  *
@@ -67,8 +67,6 @@ static struct ocpfilehandle_t *cdfs_audio_open (struct ocpfile_t *);
 static void cdfs_filehandle_ref (struct ocpfilehandle_t *);
 static void cdfs_filehandle_unref (struct ocpfilehandle_t *);
 static int cdfs_filehandle_seek_set (struct ocpfilehandle_t *, int64_t pos);
-static int cdfs_filehandle_seek_cur (struct ocpfilehandle_t *, int64_t pos);
-static int cdfs_filehandle_seek_end (struct ocpfilehandle_t *, int64_t pos);
 static uint64_t cdfs_filehandle_getpos (struct ocpfilehandle_t *);
 static int cdfs_filehandle_eof (struct ocpfilehandle_t *);
 static int cdfs_filehandle_error (struct ocpfilehandle_t *);
@@ -94,12 +92,12 @@ struct cdfs_instance_filehandle_t
 	int cursectorsize;   /* the data-size of the current sector (sectorskip is taken into account) */
 };
 
-int __attribute__ ((visibility ("internal")))
-    detect_isofile_sectorformat (struct ocpfilehandle_t *isofile_fh,
-                                 const char *filename,
-                                 off_t st_size,
-                                 enum cdfs_format_t *isofile_format,
-                                 uint32_t *isofile_sectorcount)
+OCP_INTERNAL int
+detect_isofile_sectorformat (struct ocpfilehandle_t *isofile_fh,
+                             const char *filename,
+                             off_t st_size,
+                             enum cdfs_format_t *isofile_format,
+                             uint32_t *isofile_sectorcount)
 {
 	uint8_t buffer[6+8+4+12];
 
@@ -332,7 +330,7 @@ int __attribute__ ((visibility ("internal")))
 	return 1;
 }
 
-int __attribute__ ((visibility ("internal"))) cdfs_fetch_absolute_sector_2048 (struct cdfs_disc_t *disc, uint32_t sector, uint8_t *buffer) /* 2048 byte modes */
+OCP_INTERNAL int cdfs_fetch_absolute_sector_2048 (struct cdfs_disc_t *disc, uint32_t sector, uint8_t *buffer) /* 2048 byte modes */
 {
 	int i;
 	uint8_t xbuffer[16];
@@ -346,7 +344,7 @@ int __attribute__ ((visibility ("internal"))) cdfs_fetch_absolute_sector_2048 (s
 			uint32_t relsector = sector - disc->datasources_data[i].sectoroffset;
 			if (!disc->datasources_data[i].fh)
 			{
-				bzero (buffer, 2048);
+				memset (buffer, 0, 2048);
 				return 0;
 			}
 
@@ -514,7 +512,7 @@ int __attribute__ ((visibility ("internal"))) cdfs_fetch_absolute_sector_2048 (s
 	return 1;
 }
 
-int __attribute__ ((visibility ("internal"))) cdfs_fetch_absolute_sector_2352 (struct cdfs_disc_t *disc, uint32_t sector, uint8_t *buffer)
+OCP_INTERNAL int cdfs_fetch_absolute_sector_2352 (struct cdfs_disc_t *disc, uint32_t sector, uint8_t *buffer)
 {
 	int i;
 	int subchannel = 0;
@@ -528,7 +526,7 @@ int __attribute__ ((visibility ("internal"))) cdfs_fetch_absolute_sector_2352 (s
 
 			if (!disc->datasources_data[i].fh)
 			{
-				bzero (buffer, 2352);
+				memset (buffer, 0, 2352);
 				return 0;
 			}
 
@@ -641,15 +639,15 @@ int __attribute__ ((visibility ("internal"))) cdfs_fetch_absolute_sector_2352 (s
 
 }
 
-void __attribute__ ((visibility ("internal")))
-     cdfs_disc_datasource_append (struct cdfs_disc_t     *disc,
-                                  uint32_t                sectoroffset,
-                                  uint32_t                sectorcount,
-                                  struct ocpfile_t       *file,
-                                  struct ocpfilehandle_t *fh,
-                                  enum cdfs_format_t      format,
-                                  uint64_t                offset,
-                                  uint64_t                length)
+OCP_INTERNAL void
+cdfs_disc_datasource_append (struct cdfs_disc_t     *disc,
+                             uint32_t                sectoroffset,
+                             uint32_t                sectorcount,
+                             struct ocpfile_t       *file,
+                             struct ocpfilehandle_t *fh,
+                             enum cdfs_format_t      format,
+                             uint64_t                offset,
+                             uint64_t                length)
 {
 	struct cdfs_datasource_t *temp;
 
@@ -691,17 +689,17 @@ void __attribute__ ((visibility ("internal")))
 	disc->datasources_count++;
 }
 
-void __attribute__ ((visibility ("internal")))
-     cdfs_disc_track_append (struct cdfs_disc_t *disc,
-                             uint32_t            pregap,
-                             uint32_t            start,
-                             uint32_t            length,
-                             const char         *title,
-                             const char         *performer,
-                             const char         *songwriter,
-                             const char         *composer,
-                             const char         *arranger,
-                             const char         *message)
+OCP_INTERNAL void
+cdfs_disc_track_append (struct cdfs_disc_t *disc,
+                        uint32_t            pregap,
+                        uint32_t            start,
+                        uint32_t            length,
+                        const char         *title,
+                        const char         *performer,
+                        const char         *songwriter,
+                        const char         *composer,
+                        const char         *arranger,
+                        const char         *message)
 {
 	if (disc->tracks_count >= 100)
 	{
@@ -722,7 +720,7 @@ void __attribute__ ((visibility ("internal")))
 	disc->tracks_count++;
 }
 
-enum cdfs_format_t __attribute__ ((visibility ("internal"))) cdfs_get_sector_format (struct cdfs_disc_t *disc, uint32_t sector)
+OCP_INTERNAL enum cdfs_format_t cdfs_get_sector_format (struct cdfs_disc_t *disc, uint32_t sector)
 {
 	int i;
 	for (i=0; i < disc->datasources_count; i++)
@@ -736,7 +734,7 @@ enum cdfs_format_t __attribute__ ((visibility ("internal"))) cdfs_get_sector_for
 	return FORMAT_ERROR;
 }
 
-struct cdfs_disc_t __attribute__ ((visibility ("internal"))) *cdfs_disc_new (struct ocpfile_t *file)
+OCP_INTERNAL struct cdfs_disc_t *cdfs_disc_new (struct ocpfile_t *file)
 {
 	struct cdfs_disc_t *disc;
 
@@ -768,7 +766,8 @@ struct cdfs_disc_t __attribute__ ((visibility ("internal"))) *cdfs_disc_new (str
 	                file->dirdb_ref,
 	                0, /* refcount */
 	                1, /* is_archive */
-	                0  /* is_playlist */);
+	                0, /* is_playlist */
+	                file->compression);
 
 	file->parent->ref (file->parent);
 	disc->dirs[0]->owner = disc;
@@ -866,8 +865,8 @@ static void cdfs_disc_free (struct cdfs_disc_t *disc)
 }
 
 /* returns 0 on errors */
-uint32_t __attribute__ ((visibility ("internal")))
-         CDFS_Directory_add (struct cdfs_disc_t *self, const uint32_t dir_parent_handle, const char *Dirname)
+OCP_INTERNAL uint32_t
+CDFS_Directory_add (struct cdfs_disc_t *self, const uint32_t dir_parent_handle, const char *Dirname)
 {
 	uint32_t *prev, iter;
 	uint32_t dirdb_ref;
@@ -912,7 +911,8 @@ uint32_t __attribute__ ((visibility ("internal")))
 	                dirdb_ref,
 	                0, /* refcount */
 	                1, /* is_archive */
-	                0  /* is_playlist */);
+	                0, /* is_playlist */
+	                self->dirs[0]->head.compression);
 
 	self->dirs[self->dir_fill]->owner = self;
 	self->dirs[self->dir_fill]->dir_parent = dir_parent_handle;
@@ -934,11 +934,11 @@ uint32_t __attribute__ ((visibility ("internal")))
 }
 
 /* returns UINT32_MAX on errors */
-uint32_t __attribute__ ((visibility ("internal")))
-         CDFS_File_add (struct cdfs_disc_t *self,
-                        const uint32_t      dir_parent_handle,
-                        /*char             *Filepath,*/
-                        char               *Filename)
+OCP_INTERNAL uint32_t
+CDFS_File_add (struct cdfs_disc_t *self,
+               const uint32_t      dir_parent_handle,
+               /*char             *Filepath,*/
+               char               *Filename)
 {
 	uint32_t *prev, iter;
 	uint32_t dirdb_ref;
@@ -978,7 +978,8 @@ uint32_t __attribute__ ((visibility ("internal")))
 	                 0, /* filename_override */
 	                 dirdb_ref,
 	                 0, /* refcount */
-	                 0  /* is_nodetect */);
+	                 0, /* is_nodetect */
+	                 COMPRESSION_ADD_STORE(self->dirs[0]->head.compression));
 
 	self->files[self->file_fill]->owner      = self;
 	self->files[self->file_fill]->dir_parent = dir_parent_handle;
@@ -1002,8 +1003,8 @@ uint32_t __attribute__ ((visibility ("internal")))
 	return *prev;
 }
 
-void __attribute__ ((visibility ("internal")))
-     CDFS_File_zeroextent (struct cdfs_disc_t *disc, uint32_t handle, uint64_t length)
+OCP_INTERNAL void
+CDFS_File_zeroextent (struct cdfs_disc_t *disc, uint32_t handle, uint64_t length)
 {
 	void *temp;
 	struct cdfs_instance_file_t *f;
@@ -1035,7 +1036,8 @@ append:
 	f->extents++;
 }
 
-void __attribute__ ((visibility ("internal"))) CDFS_File_extent (struct cdfs_disc_t *disc, uint32_t handle, uint32_t location, uint64_t length, int sector_skip)
+OCP_INTERNAL void
+CDFS_File_extent (struct cdfs_disc_t *disc, uint32_t handle, uint32_t location, uint64_t length, int sector_skip)
 {
 	void *temp;
 	struct cdfs_instance_file_t *f;
@@ -1071,8 +1073,8 @@ append:
 	f->extents++;
 }
 
-uint32_t __attribute__ ((visibility ("internal")))
-         CDFS_File_add_audio (struct cdfs_disc_t *self, const uint32_t dir_parent_handle, char *FilenameShort, char *FilenameLong, uint_fast32_t filesize, int audiotrack)
+OCP_INTERNAL uint32_t
+CDFS_File_add_audio (struct cdfs_disc_t *self, const uint32_t dir_parent_handle, char *FilenameShort, char *FilenameLong, uint_fast32_t filesize, int audiotrack)
 {
 	uint32_t *prev, iter;
 
@@ -1113,7 +1115,8 @@ uint32_t __attribute__ ((visibility ("internal")))
 	                 cdfs_file_filename_override,
 	                 dirdb_ref,
 	                 0, /* refcount */
-	                 0  /* is_nodetect */);
+	                 0, /* is_nodetect */
+	                 COMPRESSION_ADD_STORE(self->dirs[0]->head.compression));
 
 	self->files[self->file_fill]->owner      = self;
 	self->files[self->file_fill]->dir_parent = dir_parent_handle;
@@ -1406,8 +1409,6 @@ static struct ocpfilehandle_t *cdfs_file_open (struct ocpfile_t *_self)
 	                       cdfs_filehandle_unref,
 	                       _self,
 	                       cdfs_filehandle_seek_set,
-	                       cdfs_filehandle_seek_cur,
-	                       cdfs_filehandle_seek_end,
 	                       cdfs_filehandle_getpos,
 	                       cdfs_filehandle_eof,
 	                       cdfs_filehandle_error,
@@ -1416,15 +1417,18 @@ static struct ocpfilehandle_t *cdfs_file_open (struct ocpfile_t *_self)
 	                       cdfs_filehandle_filesize,
 	                       cdfs_filehandle_filesize_ready,
 	                       0, /* filename_override */
-	                       dirdbRef (self->head.dirdb_ref, dirdb_use_filehandle));
+	                       dirdbRef (self->head.dirdb_ref, dirdb_use_filehandle),
+	                       1 /* refcount */
+	);
+
+	cdfs_disc_ref (self->owner);
 
 	retval->file = self;
 	retval->curextent = 0;
 	retval->cursector = 0;
 	retval->curoffset = UINT64_MAX;
 
-	debug_printf ("We just created a CDFS handle, REF it\n");
-	retval->head.ref (&retval->head);
+	debug_printf ("We just created a CDFS handle\n");
 
 	return &retval->head;
 }
@@ -1440,8 +1444,6 @@ static struct ocpfilehandle_t *cdfs_audio_open (struct ocpfile_t *_self)
 	                       cdfs_filehandle_unref,
 	                       _self,
 	                       cdfs_filehandle_seek_set,
-	                       cdfs_filehandle_seek_cur,
-	                       cdfs_filehandle_seek_end,
 	                       cdfs_filehandle_getpos,
 	                       cdfs_filehandle_eof,
 	                       cdfs_filehandle_error,
@@ -1450,15 +1452,18 @@ static struct ocpfilehandle_t *cdfs_audio_open (struct ocpfile_t *_self)
 	                       cdfs_filehandle_filesize,
 	                       cdfs_filehandle_filesize_ready,
 	                       cdfs_filehandle_filename_override,
-	                       dirdbRef (self->head.dirdb_ref, dirdb_use_filehandle));
+	                       dirdbRef (self->head.dirdb_ref, dirdb_use_filehandle),
+	                       1 /* refcount */
+	);
+
+	cdfs_disc_ref (self->owner);
 
 	retval->file = self;
 	retval->curextent = 0;
 	retval->cursector = 0;
 	retval->curoffset = UINT64_MAX;
 
-	debug_printf ("We just created a CDFS audio handle, REF it\n");
-	retval->head.ref (&retval->head);
+	debug_printf ("We just created a CDFS audio handle\n");
 
 	return &retval->head;
 }
@@ -1556,7 +1561,7 @@ again:
 		/* fill the buffer */
 		if (self->file->extent[self->curextent].location == UINT32_MAX)
 		{
-			bzero (self->buffer, sizeof (self->buffer));
+			memset (self->buffer, 0, sizeof (self->buffer));
 		} else {
 			debug_printf ("CDFS FileHandle about to request physical sector %"PRIu32"\n", self->file->extent[self->curextent].location + self->cursector);
 			if (cdfs_fetch_absolute_sector_2048 (self->file->owner, self->file->extent[self->curextent].location + self->cursector, self->buffer))
@@ -1622,10 +1627,6 @@ static void cdfs_filehandle_ref (struct ocpfilehandle_t *_self)
 	struct cdfs_instance_filehandle_t *self = (struct cdfs_instance_filehandle_t *)_self;
 
 	debug_printf ("cdfs_filehandle_ref (old count = %d)\n", self->head.refcount);
-	if (!self->head.refcount)
-	{
-		cdfs_disc_ref (self->file->owner);
-	}
 	self->head.refcount++;
 }
 
@@ -1659,45 +1660,6 @@ static int cdfs_filehandle_seek_set (struct ocpfilehandle_t *_self, int64_t pos)
 	return 0;
 }
 
-static int cdfs_filehandle_seek_cur (struct ocpfilehandle_t *_self, int64_t pos)
-{
-	struct cdfs_instance_filehandle_t *self = (struct cdfs_instance_filehandle_t *)_self;
-
-	if (pos <= 0)
-	{
-		if (pos == INT64_MIN) return -1; /* we never have files this size */
-		if ((-pos) > self->filepos) return -1;
-		self->filepos += pos;
-	} else {
-		/* check for overflow */
-		if ((int64_t)(pos + self->filepos) < 0) return -1;
-
-		if ((pos + self->filepos) > self->file->filesize) return -1;
-		self->filepos += pos;
-	}
-
-	self->error = 0;
-
-	return 0;
-}
-
-static int cdfs_filehandle_seek_end (struct ocpfilehandle_t *_self, int64_t pos)
-{
-	struct cdfs_instance_filehandle_t *self = (struct cdfs_instance_filehandle_t *)_self;
-
-	if (pos > 0) return -1;
-
-	if (pos == INT64_MIN) return -1; /* we never have files this size */
-
-	if (pos < -(int64_t)(self->file->filesize)) return -1;
-
-	self->filepos = self->file->filesize + pos;
-
-	self->error = 0;
-
-	return 0;
-}
-
 static uint64_t cdfs_filehandle_getpos (struct ocpfilehandle_t *_self)
 {
 	struct cdfs_instance_filehandle_t *self = (struct cdfs_instance_filehandle_t *)_self;
@@ -1826,7 +1788,7 @@ static int cdfs_filehandle_audio_ioctl (struct ocpfilehandle_t *_self, const cha
 	return -1;
 }
 
-void __attribute__ ((visibility ("internal"))) cdfs_disc_ref (struct cdfs_disc_t *self)
+OCP_INTERNAL void cdfs_disc_ref (struct cdfs_disc_t *self)
 {
 
 	debug_printf ( " cdfs_disc_ref (old count = %d)\n", self->refcount);
@@ -1838,7 +1800,7 @@ void __attribute__ ((visibility ("internal"))) cdfs_disc_ref (struct cdfs_disc_t
 
 }
 
-void __attribute__ ((visibility ("internal"))) cdfs_disc_unref (struct cdfs_disc_t *self)
+OCP_INTERNAL void cdfs_disc_unref (struct cdfs_disc_t *self)
 {
 	debug_printf ( " cdfs_disc_unref (old count = %d)\n", self->refcount);
 
