@@ -1,7 +1,7 @@
 /*
  * This file is part of libsidplayfp, a SID player engine.
  *
- * Copyright 2011-2023 Leandro Nini <drfiemost@users.sourceforge.net>
+ * Copyright 2011-2025 Leandro Nini <drfiemost@users.sourceforge.net>
  * Copyright 2007-2010 Antti Lankila
  * Copyright 2000-2001 Simon White
  *
@@ -49,10 +49,13 @@ const char TXT_NTSC_UNKNOWN[]   = "UNKNOWN (NTSC)";
 
 // Error Strings
 const char ERR_NA[]                   = "NA";
+const char ERR_NO_TUNE_LOADED[]       = "SIDPLAYER ERROR: No tune loaded";
 const char ERR_UNSUPPORTED_FREQ[]     = "SIDPLAYER ERROR: Unsupported sampling frequency.";
 const char ERR_UNSUPPORTED_SID_ADDR[] = "SIDPLAYER ERROR: Unsupported SID address.";
 const char ERR_UNSUPPORTED_SIZE[]     = "SIDPLAYER ERROR: Size of music data exceeds C64 memory.";
 const char ERR_INVALID_PERCENTAGE[]   = "SIDPLAYER ERROR: Percentage value out of range.";
+const char ERR_BAD_BUF_SIZE[]         = "SIDPLAYER ERROR: Bad buffer size";
+const char ERR_INVALID_CONF[]         = "SIDPLAYER ERROR: Invalid configuration";
 
 /**
  * Configuration error exception.
@@ -71,7 +74,6 @@ Player::Player() :
     // Set default settings for system
     m_tune(nullptr),
     m_errorString(ERR_NA),
-    m_isPlaying(state_t::STOPPED),
     m_rand((unsigned int)std::time(nullptr))
 {
     // We need at least some minimal interrupt handling
@@ -115,27 +117,14 @@ void Player::setChargen(const uint8_t* rom)
     m_c64.getMemInterface().setChargen(rom);
 }
 
-bool Player::fastForward(unsigned int percent)
-{
-    if (!m_mixer.setFastForward(percent / 100))
-    {
-        m_errorString = ERR_INVALID_PERCENTAGE;
-        return false;
-    }
-
-    return true;
-}
-
 void Player::initialise()
 {
-    m_isPlaying = state_t::STOPPED;
-
     m_c64.reset();
 
     const SidTuneInfo* tuneInfo = m_tune->getInfo();
 
     const uint_least32_t size = static_cast<uint_least32_t>(tuneInfo->loadAddr()) + tuneInfo->c64dataLen() - 1;
-    if (size > 0xffff)
+    if (size > 0xffff) UNLIKELY
     {
         throw configError(ERR_UNSUPPORTED_SIZE);
     }
@@ -147,9 +136,23 @@ void Player::initialise()
         powerOnDelay = (uint_least16_t)((m_rand.next() >> 3) & SidConfig::MAX_POWER_ON_DELAY);
     }
 
+    powerOnDelay += 8000;
+
+    // Run for ~ [25000,50000] cycles
+    for (int i = 0; i < powerOnDelay; i++)
+    {
+        for (int j = 0; j < 3; j++)
+            m_c64.clock();
+
+        for (sidemu* chip: m_chips)
+        {
+            chip->clock();
+            chip->bufferpos(0);
+        }
+    }
+
     psiddrv driver(m_tune->getInfo());
-    driver.powerOnDelay(powerOnDelay);
-    if (!driver.drvReloc())
+    if (!driver.drvReloc()) UNLIKELY
     {
         throw configError(driver.errorString());
     }
@@ -158,157 +161,141 @@ void Player::initialise()
     m_info.m_driverLength = driver.driverLength();
     m_info.m_powerOnDelay = powerOnDelay;
 
-    driver.install(m_c64.getMemInterface(), videoSwitch);
+    driver.install(m_c64.getMemInterface(), m_videoSwitch);
 
-    if (!m_tune->placeSidTuneInC64mem(m_c64.getMemInterface()))
+    if (!m_tune->placeSidTuneInC64mem(m_c64.getMemInterface())) UNLIKELY
     {
         throw configError(m_tune->statusString());
     }
 
     m_c64.resetCpu();
+#if 0
+    // Run for some cycles until the initialization routine is done
+    for (int j = 0; j < 50; j++)
+        m_c64.clock();
+
+    for (sidemu* chip: m_chips)
+    {
+        chip->clock();
+        chip->bufferpos(0);
+    }
+#endif
+
+    m_startTime = m_c64.getTimeMs();
 }
 
 bool Player::load(SidTune *tune)
 {
     m_tune = tune;
 
-    if (tune != nullptr)
+    if (tune != nullptr) UNLIKELY
     {
         // Must re-configure on fly for stereo support!
-        if (!config(m_cfg, true))
+        if (!config(m_cfg, true)) UNLIKELY
         {
             // Failed configuration with new tune, reject it
             m_tune = nullptr;
             return false;
         }
     }
+
     return true;
 }
 
 bool Player::getSidStatus(unsigned int sidNum, uint8_t registers[32], uint8_t &volume_a, uint8_t &volume_b, uint8_t &volume_c)
 {
-    sidemu *s = m_mixer.getSid(sidNum);
-    if (s == nullptr)
+    if (sidNum < m_chips.size())
     {
-        return false;
+        sidemu *s = m_chips[sidNum];
+        s->getStatus((uint8_t *)registers);
+        s->GetVolumes(volume_a, volume_b, volume_c);
+        return true;
     }
-    s->getStatus((uint8_t *)registers);
-    s->GetVolumes(volume_a, volume_b, volume_c);
-    return true;
+    return false;
 }
 
 void Player::mute(unsigned int sidNum, unsigned int voice, bool enable)
 {
-    sidemu *s = m_mixer.getSid(sidNum);
-    if (s != nullptr)
+    if (sidemu *s = (sidNum < m_chips.size()) ? m_chips[sidNum] : nullptr)
         s->voice(voice, enable);
 }
 
 void Player::filter(unsigned int sidNum, bool enable)
 {
-    sidemu *s = m_mixer.getSid(sidNum);
-    if (s != nullptr)
+    if (sidemu *s = (sidNum < m_chips.size()) ? m_chips[sidNum] : nullptr)
         s->filter(enable);
 }
 
-/**
- * @throws MOS6510::haltInstruction
- */
-void Player::run(unsigned int events)
+void Player::initMixer(bool stereo)
 {
-    for (unsigned int i = 0; (m_isPlaying != state_t::STOPPED) && (i < events); i++)
-        m_c64.clock();
+    std::unique_ptr<short*[]> bufs(new short*[m_chips.size()]);
+    buffers(bufs.get());
+    m_simpleMixer.reset(new SimpleMixer(stereo, bufs.get(), installedSIDs()));
 }
 
-uint_least32_t Player::play(int16_t *buffer, uint_least32_t count, std::vector<int16_t*> *rawBuffers)
+unsigned int Player::mix(short *buffer, unsigned int samples, std::vector<int16_t*> *rawBuffers)
 {
-    static constexpr unsigned int CYCLES = 3000;
+    return m_simpleMixer->doMix(buffer, samples, rawBuffers);
+}
 
+void Player::buffers(short** buffers) const
+{
+    for (size_t i = 0; i < m_chips.size(); i++)
+    {
+        buffers[i] = m_chips[i]->buffer();
+    }
+}
+
+int Player::play(unsigned int cycles)
+{
     // Make sure a tune is loaded
-    if (m_tune == nullptr)
-        return 0;
-
-    // Start the player loop
-    if (m_isPlaying == state_t::STOPPED)
-        m_isPlaying = state_t::PLAYING;
-
-    if (m_isPlaying == state_t::PLAYING)
+    if (m_tune == nullptr) UNLIKELY
     {
-        try
-        {
-            m_mixer.begin(buffer, count, rawBuffers);
-
-            if (m_mixer.getSid(0) != nullptr)
-            {
-                if (count && (buffer != nullptr))
-                {
-                    // reset count in case of exceptions
-                    count = 0;
-
-                    // Clock chips and mix into output buffer
-                    while ((m_isPlaying != state_t::STOPPED) && m_mixer.notFinished())
-                    {
-                        if (!m_mixer.wait())
-                            run(CYCLES);
-
-                        m_mixer.clockChips();
-                        m_mixer.doMix();
-                    }
-                    count = m_mixer.samplesGenerated();
-                }
-                else
-                {
-                    // Clock chips and discard buffers
-                    int size = m_c64.getMainCpuSpeed() / m_cfg.frequency;
-                    while ((m_isPlaying != state_t::STOPPED) && --size)
-                    {
-                        run(CYCLES);
-
-                        m_mixer.clockChips();
-                        m_mixer.resetBufs();
-                    }
-                }
-            }
-            else
-            {
-                // Clock the machine
-                int size = m_c64.getMainCpuSpeed() / m_cfg.frequency;
-                while ((m_isPlaying != state_t::STOPPED) && --size)
-                {
-                    run(CYCLES);
-                }
-            }
-        }
-        catch (MOS6510::haltInstruction const &)
-        {
-            m_errorString = "Illegal instruction executed";
-            m_isPlaying = state_t::STOPPING;
-        }
-        catch (Mixer::badBufferSize const &)
-        {
-            m_errorString = "Bad buffer size";
-            m_isPlaying = state_t::STOPPING;
-        }
+        m_errorString = ERR_NO_TUNE_LOADED;
+        return -1;
     }
 
-    if (m_isPlaying == state_t::STOPPING)
+    // Limit to roughly 20ms
+    constexpr unsigned int max_cycles = 20000;
+    if (cycles > max_cycles)
     {
-        try
-        {
-            initialise();
-        }
-        catch (configError const &) {}
-        m_isPlaying = state_t::STOPPED;
+        cycles = max_cycles;
     }
 
-    return count;
+    try
+    {
+        for (unsigned int i = 0; i < cycles; i++)
+            m_c64.clock();
+
+        int sampleCount = 0;
+        for (sidemu *s: m_chips)
+        {
+            // clock the chip and get the buffer
+            // buffersize is expected to be the same
+            // for all chips
+            s->clock();
+            sampleCount = s->bufferpos();
+            // Reset the buffer
+            s->bufferpos(0);
+        }
+        return sampleCount;
+    }
+    catch (MOS6510::haltInstruction const &ill)
+    {
+        m_errorString = ill.message();
+        return -1;
+    }
 }
 
-void Player::stop()
+bool Player::reset()
 {
-    if ((m_tune != nullptr) && (m_isPlaying == state_t::PLAYING))
+    try
     {
-        m_isPlaying = state_t::STOPPING;
+        initialise();
+        return true;
+    }
+    catch (configError const &) {
+        return false;
     }
 }
 
@@ -331,8 +318,8 @@ bool Player::config(const SidConfig &cfg, bool force)
         return true;
     }
 
-    // Check for base sampling frequency
-    if (cfg.frequency < 8000)
+    // Check for a sane sampling frequency
+    if ((cfg.frequency < 8000) || (cfg.frequency > 192000)) UNLIKELY
     {
         m_errorString = ERR_UNSUPPORTED_FREQ;
         return false;
@@ -371,7 +358,7 @@ bool Player::config(const SidConfig &cfg, bool force)
             const c64::cia_model_t ciaModel = getCiaModel(cfg.ciaModel);
             m_c64.setCiaModel(ciaModel);
 
-            sidParams(m_c64.getMainCpuSpeed(), cfg.frequency, cfg.samplingMethod, cfg.fastSampling);
+            sidParams(m_c64.getMainCpuSpeed(), cfg.frequency, cfg.samplingMethod);
 
             // Configure, setup and install C64 environment/events
             initialise();
@@ -392,10 +379,6 @@ bool Player::config(const SidConfig &cfg, bool force)
     const bool isStereo = cfg.playback == SidConfig::STEREO;
     m_info.m_channels = isStereo ? 2 : 1;
 
-    m_mixer.setStereo(isStereo);
-    m_mixer.setSamplerate(cfg.frequency);
-    m_mixer.setVolume(cfg.leftVolume, cfg.rightVolume);
-
     // Update Configuration
     m_cfg = cfg;
 
@@ -419,27 +402,27 @@ c64::model_t Player::c64model(SidConfig::c64_model_t defaultModel, bool forced)
         case SidConfig::PAL:
             clockSpeed = SidTuneInfo::CLOCK_PAL;
             model = c64::PAL_B;
-            videoSwitch = 1;
+            m_videoSwitch = 1;
             break;
         case SidConfig::DREAN:
             clockSpeed = SidTuneInfo::CLOCK_PAL;
             model = c64::PAL_N;
-            videoSwitch = 1; // TODO verify
+            m_videoSwitch = 1; // TODO verify
             break;
         case SidConfig::NTSC:
             clockSpeed = SidTuneInfo::CLOCK_NTSC;
             model = c64::NTSC_M;
-            videoSwitch = 0;
+            m_videoSwitch = 0;
             break;
         case SidConfig::OLD_NTSC:
             clockSpeed = SidTuneInfo::CLOCK_NTSC;
             model = c64::OLD_NTSC_M;
-            videoSwitch = 0;
+            m_videoSwitch = 0;
             break;
         case SidConfig::PAL_M:
             clockSpeed = SidTuneInfo::CLOCK_NTSC;
             model = c64::PAL_M;
-            videoSwitch = 0; // TODO verify
+            m_videoSwitch = 0; // TODO verify
             break;
         }
     }
@@ -450,11 +433,11 @@ c64::model_t Player::c64model(SidConfig::c64_model_t defaultModel, bool forced)
         default:
         case SidTuneInfo::CLOCK_PAL:
             model = c64::PAL_B;
-            videoSwitch = 1;
+            m_videoSwitch = 1;
             break;
         case SidTuneInfo::CLOCK_NTSC:
             model = c64::NTSC_M;
-            videoSwitch = 0;
+            m_videoSwitch = 0;
             break;
         }
     }
@@ -484,6 +467,19 @@ c64::model_t Player::c64model(SidConfig::c64_model_t defaultModel, bool forced)
     return model;
 }
 
+SidTuneInfo::model_t getSidModel(SidConfig::sid_model_t sidModel)
+{
+    switch (sidModel)
+    {
+    case SidConfig::MOS6581:
+        return SidTuneInfo::SIDMODEL_6581;
+    case SidConfig::MOS8580:
+        return SidTuneInfo::SIDMODEL_8580;
+    default:
+        return SidTuneInfo::SIDMODEL_UNKNOWN;
+    }
+}
+
 /**
  * Get the SID model.
  *
@@ -491,7 +487,7 @@ c64::model_t Player::c64model(SidConfig::c64_model_t defaultModel, bool forced)
  * @param defaultModel the default model
  * @param forced true if the default model shold be forced in spite of tune model
  */
-SidConfig::sid_model_t makeSidModel(SidTuneInfo::model_t sidModel, SidConfig::sid_model_t defaultModel, bool forced)
+SidConfig::sid_model_t getSidModel(SidTuneInfo::model_t sidModel, SidConfig::sid_model_t defaultModel, bool forced)
 {
     SidTuneInfo::model_t tuneModel = sidModel;
 
@@ -511,39 +507,29 @@ SidConfig::sid_model_t makeSidModel(SidTuneInfo::model_t sidModel, SidConfig::si
         }
     }
 
-    SidConfig::sid_model_t newModel;
-
     switch (tuneModel)
     {
     default:
     case SidTuneInfo::SIDMODEL_6581:
-        newModel = SidConfig::MOS6581;
-        break;
+        return SidConfig::MOS6581;
     case SidTuneInfo::SIDMODEL_8580:
-        newModel = SidConfig::MOS8580;
-        break;
+        return SidConfig::MOS8580;
     }
-
-    return newModel;
 }
 
 void Player::sidRelease()
 {
     m_c64.clearSids();
 
-    for (unsigned int i = 0; ; i++)
+    for (sidemu *s: m_chips)
     {
-        sidemu *s = m_mixer.getSid(i);
-        if (s == nullptr)
-            break;
-
         if (sidbuilder *b = s->builder())
         {
             b->unlock(s);
         }
     }
 
-    m_mixer.clearSids();
+    m_chips.clear();
 }
 
 void Player::sidCreate(sidbuilder *builder, SidConfig::sid_model_t defaultModel, bool digiboost,
@@ -551,23 +537,24 @@ void Player::sidCreate(sidbuilder *builder, SidConfig::sid_model_t defaultModel,
 {
     if (builder != nullptr)
     {
+        m_chips.clear();
+        m_info.m_sidModels.clear();
+        m_info.m_sidAddresses.clear();
         const SidTuneInfo* tuneInfo = m_tune->getInfo();
 
         // Setup base SID
-        const SidConfig::sid_model_t userModel = makeSidModel(tuneInfo->sidModel(0), defaultModel, forced);
+        const SidConfig::sid_model_t userModel = getSidModel(tuneInfo->sidModel(0), defaultModel, forced);
         sidemu *s = builder->lock(m_c64.getEventScheduler(), userModel, digiboost);
-        if (!builder->getStatus())
+        if (!s) UNLIKELY
         {
             throw configError(builder->error());
         }
 
-        m_sidModels.clear();
-        m_sidAddresses.clear();
-        m_sidModels.push_back(userModel);
-        m_sidAddresses.push_back(0xd400);
 
         m_c64.setBaseSid(s);
-        m_mixer.addSid(s);
+        m_chips.push_back(s);
+        m_info.m_sidModels.push_back(getSidModel(userModel));
+        m_info.m_sidAddresses.push_back(0xd400);
 
         // Setup extra SIDs if needed
         if (extraSidAddresses.size() != 0)
@@ -580,46 +567,40 @@ void Player::sidCreate(sidbuilder *builder, SidConfig::sid_model_t defaultModel,
 
             for (unsigned int i = 0; i < extraSidChips; i++)
             {
-                const SidConfig::sid_model_t userModel = makeSidModel(tuneInfo->sidModel(i+1), defaultModel, forced);
+                const SidConfig::sid_model_t userModel = getSidModel(tuneInfo->sidModel(i+1), defaultModel, forced);
 
                 sidemu *s = builder->lock(m_c64.getEventScheduler(), userModel, digiboost);
-                if (!builder->getStatus())
+                if (!s) UNLIKELY
                 {
                     throw configError(builder->error());
                 }
 
-                m_sidModels.push_back(userModel);
-                m_sidAddresses.push_back(extraSidAddresses[i]);
-                if (!m_c64.addExtraSid(s, extraSidAddresses[i]))
+                if (!m_c64.addExtraSid(s, extraSidAddresses[i])) UNLIKELY
                     throw configError(ERR_UNSUPPORTED_SID_ADDR);
 
-                m_mixer.addSid(s);
+                m_chips.push_back(s);
+                m_info.m_sidModels.push_back(getSidModel(userModel));
+                m_info.m_sidAddresses.push_back(extraSidAddresses[i]);
             }
         }
     }
 }
 
 void Player::sidParams(double cpuFreq, int frequency,
-                        SidConfig::sampling_method_t sampling, bool fastSampling)
+                        SidConfig::sampling_method_t sampling)
 {
-    for (unsigned int i = 0; ; i++)
+    for (sidemu *s: m_chips)
     {
-        sidemu *s = m_mixer.getSid(i);
-        if (s == nullptr)
-            break;
-
-        s->sampling((float)cpuFreq, frequency, sampling, fastSampling);
+        s->sampling((float)cpuFreq, frequency, sampling);
     }
 }
 
 bool Player::getSidStatus(unsigned int sidNum, uint8_t regs[32])
 {
-    sidemu *s = m_mixer.getSid(sidNum);
-
-    if (s == nullptr)
+    if (sidNum >= m_chips.size())
         return false;
 
-    s->getStatus(regs);
+    m_chips[sidNum]->getStatus(regs);
     return true;
 }
 
